#!/usr/bin/python
"""Script that checks an UBY-LMF file containing plWordNet data for consistency.

Specifically, it checks for the following errors:
    * Empty synsets.
    * Lexical units belonging to nonexistent synsets.
    * Synsets and lexical relations to nonexistent synsets / units

UBY-LMF module also prints out encountered errors, so be sure to capture the
standard output.
"""

import argparse as argp
import collections as coll
# import itertools as itt
import logging as log
import sys

from six import iteritems

import plwn.ubylmf_reader as ubyr


def main():
    ap = argp.ArgumentParser(description=__doc__)
    ap.add_argument('uby_lmf_file', help='The file to check.')

    av = ap.parse_args()
    # Log every error and warning from the plwn module along with this script's
    # output
    plwn_log = log.getLogger('plwn')
    plwn_log_h = log.StreamHandler(sys.stdout)
    plwn_log_h.setFormatter(log.Formatter('!! Log from %(name)s: %(message)s'))
    plwn_log.addHandler(plwn_log_h)
    plwn_log.setLevel(log.WARNING)

    _verify(ubyr.iterread(av.uby_lmf_file))


def _verify(uby_lmf_reader):
    synset_defs = set()
    unit_defs = set()
    synsets2units = coll.defaultdict(list)
    synrels = {}
    lexrels = {}

    for record in uby_lmf_reader:
        if isinstance(record, ubyr.SynsetNode):
            synset_defs.add(record.id)
            synrels[record.id] = frozenset(item[0] for item in record.related)
        else:
            unit_defs.add(record.id)
            lexrels[record.id] = frozenset(item[0] for item in record.related)
            synsets2units[record.synset].append(record.id)

    # Empty synsets
    print('!! Empty synsets:')
    empties = synset_defs.difference(synsets2units)
    if empties:
        for synid in empties:
            print('S' + str(synid))

    # Lexical units belonging to nonexistent synsets
    print('!! Units belonging to nonexistent synsets:')
    syn_nonexistent = unit_defs.difference(synsets2units)
    if syn_nonexistent:
        for synid in syn_nonexistent:
            for lexid in synsets2units[synid]:
                print('L' + str(lexid))

    # Synrelations to nonexistent targers
    print('!! Synset relations to nonexistent targets:')
    for parent, targets in iteritems(synrels):
        bad_targets = targets - synset_defs
        for bad in bad_targets:
            print('S{} -> S{}'.format(parent, bad))

    # Lexrelations to nonexistent synsets
    print('!! Lexical relations to nonexistent targets:')
    for parent, targets in iteritems(lexrels):
        bad_targets = targets - unit_defs
        for bad in bad_targets:
            print('L{} -> L{}'.format(parent, bad))


if __name__ == '__main__':
    main()
